#!/usr/bin/env python

import argparse, os, random


def get_args():
    parser = argparse.ArgumentParser(description='LAD-incomplete generator')

    parser.add_argument('--output', '-o', metavar='filename', type=str,
                        default=None, help='The file to write the problem to')

    parser.add_argument('--number', '-n', metavar='n', type=int,
                        default=1, help='The number of blocks of actions to generate')

    parser.add_argument('--problems', '-p', metavar='n', type=int,
                        default=10, help='The number of blocks of actions to generate')

    parser.add_argument('--random', '-r', action="store_true",
                        help='Introduce additional random dependencies " \
                        "(still guaranteeing plan existence)')

    return parser.parse_args()


def main():
    args = get_args()

    for j in xrange(args.problems):
        main_aux(j, args)

def main_aux(j, args):

    out = args.output
    if out is None:
        def get_name(i,j):
            return "p%d%d" % (i, j)
        i = 0
        while os.path.exists(get_name(i, j)):
            i += 1
        out = get_name(i,j)
    out = out + "_" + str(j)
    with open(out, "w") as f:
        def w(s): f.write(s); f.write("\n")

        w("(define (problem prob)")
        w("  (:domain lad-incomplete)")
        w("  (:objects")
        for i in xrange(args.number):
            w("    c%d - container" % i)
            w("    m%d - machine" % i)
            w("    d%d - device" % i)
        w("  )")
        w("  (:init")
        for i in xrange(args.number):
            w("    (p0 c%d)" % i)
            w("    (machine-of-device m%d d%d)" % (i, i))
            w("    (container-of-machine c%d m%d)" % (i, i))
        if args.random:
            n = random.randint(0, 2*args.number)
            for _ in xrange(n):
                s,d = random.sample(range(args.number), 2)
                if random.uniform(0,1) < 0.1:
                    w("    (machine-of-device m%d d%d)" % (s, d))
                else:
                    w("    (container-of-machine c%d m%d)" % (s, d))

        w("    (p)")
        w("    (q)")
        w("  )")
        w("  (:goal")
        w("    (and")
        for i in xrange(args.number):
            w("      (p2 c%d)" % i)
            w("      (p3 c%d)" % i)
        w("    )")
        w("  )")
        w("  (:metric minimize (total-time))")
        w(")")

if __name__ == "__main__":
    main()
